---
layout: assignment
published: true

title: Improve a screen reader
code: as3

assigned: April 5, 2021
due:
 - April 21, 2021

revised: April 21, 2021

objective:  Fix screen reader bugs; Contribute to the accessibility Community

---

* TOC
{:toc}

# Learning Goals

The goal of this activity is to
- Learn more about how screen readers work
- Be a contributor to the accessibility community
- Get direct experience with the relation between web accessibility best practices and screen reader behavior

# Overview

People who are blind or visually impaired (BVI) access computers through screen readers. Simply put, screen readers are software that render the graphical user interface in audio (synthesized speech and sounds) or Braille (using a refreshable braille display). There are different types of screen readers, that run on a variety of platforms; from desktops, to smartphones, to smart-watches to smart speakers. One such screen reader is the Non Visual Desktop Access (NVDA) screen reader, a free, open source screen reader for windows. As per the [WebAIM: Screen Reader User Survey #8 Results](https://webaim.org/projects/screenreadersurvey8/), NVDA is the most used screen reader among respondents.

In this homework, you will be working on a bug in the NVDA screen reader. Your approach should improve the usability of the NVDA screen reader for its intended audience. (remember, NVDA is developed by a huge community of developers who may be BVI so they are your target audience too!)

To reduce the complexity of contributing to an open source project in a short time frame, we curated a list of potential github issues that you could work on. You are also free to pick other open issues from the NVDA github repository. We however recommend refraining from issues with high complexity or issues where a consensus has not been reached.  You can find the full list of issues below. 

Once you decide the github issue that you would like to work on, please join the corresponding [group on Canvas](https://canvas.uw.edu/courses/1465814/groups#tab-122183). You may start working *individually* on a solution right away, but the group is there to share ideas and for phase 1's handin.


# NVDA Issues



1. [NVDA announces "Article" before every article while browsing the articles via quick keys #11103](https://github.com/nvaccess/nvda/issues/11103)
2. [Failing the checkPot step should be more obvious. #10928](https://github.com/nvaccess/nvda/issues/10928)
3. [Report status of screen curtain at startup when "Play a sound when toggling screen curtain" is disabled. · Issue #10692](https://github.com/nvaccess/nvda/issues/10692) (bug affecting screen reader announcement of screen curtain status. Picked up by a developer, but abandoned). 
4. [Add script for aria figure to input gestures and give the user the possibility to turn off reporing of this role in document formating settings · Issue #10826](https://github.com/nvaccess/nvda/issues/10826) (feature request to make NVDA announcement of “figure” optional.
5. [Shorten "out of element" message #10446](https://github.com/nvaccess/nvda/issues/10446)
6. [Selection not announced when jumping to Top/Bottom of Page · Issue #5549](https://github.com/nvaccess/nvda/issues/5549)
7. [aria-labelledby element gets read twice #10512](https://github.com/nvaccess/nvda/issues/10512) (potentially a blocker on chrome?)
8. [Grid cells with focusable controls get read twice #10096](https://github.com/nvaccess/nvda/issues/10096) (abandoned, and potentially a spec conformance but annoying behavior)
9. [Table navigation doesn't respect colspan · Issue #7278](https://github.com/nvaccess/nvda/issues/7278)

## Phase 1: Jointly prepare solution approach submission for NVDA (only for us)

You will work as a group to prepare a solution approach, proposing one or multiple ways of working on  a bug to improve the user experience for NVDA users. To do this you should all try to replicate the issue and think individually about solutions. Then summarize your results in a group-constructed document.

You will not submit these to NVDA; you will submit these proposals as a group to [this Canvas Assignment](https://canvas.uw.edu/courses/1465814/assignments/6267341) for review by the course staff. 

You will also give a  group presentation discussing:
- The issue
   - What is it
   - Why do screen reader users care?
- Reproducing the issue
   - Were any members of your group able to do so?
   - What did you learn from this
- Solutions
   - What approaches did your group come up with?
   - If you couldn't come up with an approach, why?
- What text you are proposing to submit to the NVDA Issue Tracker
- Other Lessons Learned

You will need to submit accessible slides to [this canvas assignment](https://canvas.uw.edu/courses/1465814/assignments/6148838). Please [make your presentations accessible, and present accessibly](http://www.sigaccess.org/welcome-to-sigaccess/resources/accessible-presentation-guide/) (here is [a short video](https://www.youtube.com/watch?v=L9TxhGv91kc) that provides some guidance)

## Phase 2: Your personal perspective

Please turn in an approximately 1 page write up answering the following questions

- What did you learn from trying to use NVDA?
- What issue did you address?
- Were you able to replicate it?
- What solution approach do you suggest? 
- What did you learn about how screen readers and web pages interact to make an experience more or less accessible?

## Optional Phase 3: Submitting your solution to NVDA

Your group may start communicating with NVDA after receiving approval from the staff. Please keep in mind that you should consider the opinions expressed by the NVDA user community through comments on the github issue of your choosing. 

If your approach is approved by the NVDA community, you and any other group members who use the same approach will have the opportunity to actually submit the best of your solutions. 

## NVDA setup and other pre-requisites

You will need a Windows computer running Windows 10 and python 3.7 (32-bit) to install NVDA. If you do not have a Windows machine, please reach out to us as soon as possible. We have a limited number of machines that can be accessed remotely. The instructions to build NVDA are in the [NVDA github repository readme](https://github.com/nvaccess/nvda). Please also be sure to [read the contributing guide](https://github.com/nvaccess/nvda/wiki/Contributing).

## Grading

This is a 10 point assignment.  

- Two points for the presentation (including whether it is accessible and accessibly presented)
- Two points to the group writeup (completeness)
- 6 points the individual writeup based on its demonstration of the three learning goals stated at the beginning of the assignment. 
  - Learn more about how screen readers work
  - Be a contributor to the accessibility community
  - Get direct experience with the relation between web accessibility best   practices and screen reader behavior


